#!/bin/bash

set -e

have_nvidia=`lspci -nn | grep VGA |grep -i NVIDIA | tr 'a-z' 'A-Z' `

if [ -z "$have_nvidia" ]; then
	echo "not nvidia, return"
	exit 0
fi

#nvidia_vid=`lspci -mn | awk '{ if ($2 ~ "\"030[0-9]\"") { print $4 } }' | cut -d"\"" -f2 | tr 'a-z' 'A-Z' `
#nvidia_ids=`lspci -mn | awk '{ gsub("\"",""); if ($2 ~ "030[0-9]") { print $4 } }' | tr 'a-z' 'A-Z' `
nvidia_ids=`lspci -nn | grep VGA | grep -i NVIDIA | tr 'a-z' 'A-Z' | cut -d ":" -f 4 | cut -d "]" -f 1| tr 'a-z' 'A-Z' `

if [ -z "$nvidia_ids" ]; then
	echo "Cannot find $nvidia_ids in nvidia list, so return"
	rm -rf /etc/modprobe.d/nvidia* /etc/modules-load.d/nvidia* /etc/X11/nvidia-xorg.conf
	exit 0
fi

# 第一次换Nvidia卡
FIRST_MATCHED_NVIDS="/etc/default/first-matched-nvids"
if [ -e $FIRST_MATCHED_NVIDS ]; then
	first_matched_nvids=`cat $FIRST_MATCHED_NVIDS`
	if [[ $nvidia_ids == *"$first_matched_nvids"* ]]; then
		echo "Have been matched on this Nvidia card, don't continue, so return. "
		exit 0
	fi
fi
echo "$nvidia_ids" > $FIRST_MATCHED_NVIDS

# 连接Nvidia 闭源驱动
function searchlinkNvidiaDriver() {
	echo "Start to link nvidia driver $1"

	if [ ! -d /usr/nfs-nvidia-drivers/$1 ]; then
		echo "Cannot find /usr/nfs-nvidia-drivers/$1 ."
		exit 0
	fi

	for src_lists in $(find /usr/nfs-nvidia-drivers/$1/ -type f -o -type l); do
		dest_links=`echo $src_lists | awk -F "/usr/nfs-nvidia-drivers/$1" '{print $2 }'`
		rm -rf $dest_links
		ln -sf $src_lists $dest_links
	done
	
	loadofnvidia=`echo $1 | awk -F "." '{print $1}'`
}

DEFAULT_SELECT="/etc/default/nvidia-defselect"
if [ -e $DEFAULT_SELECT ]; then
	selected_nvidia=`cat $DEFAULT_SELECT`
	if [ -z $selected_nvidia ]; then
		echo "Warning: No choose compatible drivers manually, please enter driver-menu or contact System Integrator. "
		exit 0
	fi
	for availabledrivers in $(find /usr/share/nvidia-drivers-ids/ -xtype f -iname '*.ids' | xargs grep -inr $nvidia_ids | awk -F: '{ print $1}' | cut -d "-" -f5 | sed 's/.ids$//g' | sort); do
		if [[ $selected_nvidia == *"$availabledrivers"* ]]; then
			searchlinkNvidiaDriver $selected_nvidia
			exit 0
		fi
	done
fi

nvidia_linknum=`find /usr/share/nvidia-drivers-ids/ -xtype f -iname '*.ids' | xargs grep -inr $nvidia_ids | awk -F: '{ print $1}' | cut -d "-" -f5 | sed 's/.ids$//g' | sort | cut -d "." -f1 `
nvid_max=0
for find_nvid in $nvidia_linknum; do
	DVERSION=`dpkg-query -W -f='${Version}' nvidia-driver-$find_nvid | awk -F "-" '{print $1}'`
	if [[ $DVERSION == *"$find_nvid"*  ]]; then
		if [ $find_nvid -ge $nvid_max ]
		then
			nvid_max=$find_nvid
		fi
	fi
done

DVERSION=`dpkg-query -W -f='${Version}' nvidia-driver-$nvid_max | awk -F "-" '{print $1}' `
if [[ $DVERSION == *"$nvid_max"* ]]; then
	echo "match id DVERSION = $DVERSION"
	#echo "$DVERSION" > $DEFAULT_SELECT
	searchlinkNvidiaDriver $DVERSION
else
	rm -rf /etc/modprobe.d/nvidia* /etc/modules-load.d/nvidia* /etc/X11/nvidia-xorg.conf
	echo "Error: Cannot find any matched drivers for nvidia $nvidia_linknum."
fi

exit 0
